require "typhoeus"
require "json"
require "active_support/all"

require "gql_metrics_tracer/field_tracer"
require "gql_metrics_tracer/query_tracer"
require "gql_metrics_tracer/api_client"

module GQLMetrics
  class Tracer
    def self.use(schema_definition, _options = {})
      tracer = new(schema_definition)
      schema_definition.instrument(:query, tracer)
      schema_definition.tracer(tracer)
    end

    def initialize(schema_definition)
      @api_client = ApiClient.new schema_definition: schema_definition
      Thread.new { @api_client.publish_schema! }
    end

    def before_query(query)
      query.context["gqlmetrics-trace"] = {
        "query-tracer" => QueryTracer.new(
          schema_id: schema_id,
          query: query.document.to_query_string,
          operation_name: query.context[:operation_name],
        ),
      }
    end

    def after_query(query)
      query_tracer = query.context.dig("gqlmetrics-trace", "query-tracer")
      query_tracer.end_trace!

      Thread.new { api_client.send_trace!(query_tracer) }
    end

    def trace(key, data, &block)
      if key == "execute_field"
        context = data[:context] || data[:object].context
        query_tracer = context.dig("gqlmetrics-trace", "query-tracer")

        field = !data[:context].nil? ? data[:context].field : data[:field]
        path = !data[:context].nil? ? data[:context].path : data[:path]
        parent_type = !data[:context].nil? ? data[:context].parent_type : data[:owner].graphql_name

        query_tracer.trace_field!(
          field: field,
          path: path,
          parent_type: parent_type,
          context: context,
          &block
        )
      else
        yield
      end
    end

    private

    attr_reader :api_client

    delegate :schema_id, to: :api_client
  end
end
